@@ -267,7 +267,7 @@ poppler_form_field_button_set_state (PopplerFormField *field,
 gchar*
 poppler_form_field_get_partial_name (PopplerFormField *field)
 {
-  GooString *tmp;
+  const GooString *tmp;
 
   g_return_val_if_fail (POPPLER_IS_FORM_FIELD (field), NULL);
 
@@ -290,7 +290,7 @@ poppler_form_field_get_partial_name (PopplerFormField *field)
 gchar*
 poppler_form_field_get_mapping_name (PopplerFormField *field)
 {
-  GooString *tmp;
+  const GooString *tmp;
 
   g_return_val_if_fail (POPPLER_IS_FORM_FIELD (field), NULL);
 
@@ -360,7 +360,7 @@ gchar *
 poppler_form_field_text_get_text (PopplerFormField *field)
 {
   FormWidgetText *text_field;
-  GooString      *tmp;
+  const GooString *tmp;
 
   g_return_val_if_fail (field->widget->getType () == formText, NULL);
 
@@ -571,7 +571,7 @@ gchar *
 poppler_form_field_choice_get_item (PopplerFormField *field,
 				    gint              index)
 {
-  GooString *tmp;
+  const GooString *tmp;
   
   g_return_val_if_fail (field->widget->getType () == formChoice, NULL);
   g_return_val_if_fail (index >= 0 && index < poppler_form_field_choice_get_n_items (field), NULL);
@@ -682,7 +682,7 @@ poppler_form_field_choice_set_text (PopplerFormField *field,
 gchar *
 poppler_form_field_choice_get_text (PopplerFormField *field)
 {
-  GooString *tmp;
+  const GooString *tmp;
   
   g_return_val_if_fail (field->widget->getType () == formChoice, NULL);
 
