@@ -23,8 +23,9 @@
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2013 Dominik Haumann <dhaumann@kde.org>
 // Copyright (C) 2013 Mihai Niculescu <q.quark@gmail.com>
-// Copyright (C) 2017 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2017, 2018 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -50,6 +51,9 @@
 #include "FontEncodingTables.h"
 #include <fofi/FoFiTrueType.h>
 #include "ArthurOutputDev.h"
+#include "Page.h"
+#include "Gfx.h"
+#include "PDFDoc.h"
 
 #include <QtCore/QtDebug>
 #include <QRawFont>
@@ -70,7 +74,7 @@
 class SplashOutFontFileID: public SplashFontFileID {
 public:
 
-  SplashOutFontFileID(Ref *rA) { r = *rA; }
+  SplashOutFontFileID(const Ref *rA) { r = *rA; }
 
   ~SplashOutFontFileID() {}
 
@@ -86,6 +90,86 @@ private:
 
 #endif
 
+class ArthurType3Font
+{
+public:
+
+  ArthurType3Font(PDFDoc* doc, Gfx8BitFont* font);
+
+  const QPicture& getGlyph(int gid) const;
+
+private:
+  PDFDoc* m_doc;
+  Gfx8BitFont* m_font;
+
+  mutable std::vector<std::unique_ptr<QPicture> > glyphs;
+
+public:
+  std::vector<int> codeToGID;
+};
+
+ArthurType3Font::ArthurType3Font(PDFDoc* doc, Gfx8BitFont* font)
+: m_doc(doc), m_font(font)
+{
+  char *name;
+  const Dict* charProcs = font->getCharProcs();
+
+  // Storage for the rendered glyphs
+  glyphs.resize(charProcs->getLength());
+
+  // Compute the code-to-GID map
+  char **enc = font->getEncoding();
+
+  codeToGID.resize(256);
+
+  for (int i = 0; i < 256; ++i) {
+    codeToGID[i] = 0;
+    if (charProcs && (name = enc[i])) {
+      for (int j = 0; j < charProcs->getLength(); j++) {
+        if (strcmp(name, charProcs->getKey(j)) == 0) {
+          codeToGID[i] = j;
+        }
+      }
+    }
+  }
+}
+
+const QPicture& ArthurType3Font::getGlyph(int gid) const
+{
+  if (!glyphs[gid]) {
+
+    // Glyph has not been rendered before: render it now
+
+    // Smallest box that contains all the glyphs from this font
+    const double* fontBBox = m_font->getFontBBox();
+    PDFRectangle box(fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]);
+
+    Dict* resDict = m_font->getResources();
+
+    QPainter glyphPainter;
+    glyphs[gid] = std::unique_ptr<QPicture>(new QPicture);
+    glyphPainter.begin(glyphs[gid].get());
+    std::unique_ptr<ArthurOutputDev> output_dev(new ArthurOutputDev(&glyphPainter));
+
+    std::unique_ptr<Gfx> gfx(new Gfx(m_doc, output_dev.get(), resDict,
+                                     &box,  // pagebox
+                                     nullptr));  // cropBox
+
+    output_dev->startDoc(m_doc);
+
+    output_dev->startPage (1, gfx->getState(), gfx->getXRef());
+
+    const Dict* charProcs = m_font->getCharProcs();
+    Object charProc = charProcs->getVal(gid);
+    gfx->display(&charProc);
+
+    glyphPainter.end();
+  }
+
+  return *glyphs[gid];
+}
+
+
 //------------------------------------------------------------------------
 // ArthurOutputDev
 //------------------------------------------------------------------------
@@ -106,8 +190,9 @@ ArthurOutputDev::~ArthurOutputDev()
 #endif
 }
 
-void ArthurOutputDev::startDoc(XRef *xrefA) {
-  xref = xrefA;
+void ArthurOutputDev::startDoc(PDFDoc* doc) {
+  xref = doc->getXRef();
+  m_doc = doc;
 #ifdef HAVE_SPLASH
   delete m_fontEngine;
 
@@ -123,18 +208,7 @@ void ArthurOutputDev::startDoc(XRef *xrefA) {
 }
 
 void ArthurOutputDev::startPage(int pageNum, GfxState *state, XRef *)
-{
-  // fill page with white background.
-  int w = static_cast<int>(state->getPageWidth());
-  int h = static_cast<int>(state->getPageHeight());
-  QColor fillColour(Qt::white);
-  QBrush fill(fillColour);
-  m_painter.top()->save();
-  m_painter.top()->setPen(fillColour);
-  m_painter.top()->setBrush(fill);
-  m_painter.top()->drawRect(0, 0, w, h);
-  m_painter.top()->restore();
-}
+{}
 
 void ArthurOutputDev::endPage() {
 }
@@ -144,6 +218,7 @@ void ArthurOutputDev::saveState(GfxState *state)
   m_currentPenStack.push(m_currentPen);
   m_currentBrushStack.push(m_currentBrush);
   m_rawFontStack.push(m_rawFont);
+  m_type3FontStack.push(m_currentType3Font);
   m_codeToGIDStack.push(m_codeToGID);
 
   m_painter.top()->save();
@@ -157,6 +232,8 @@ void ArthurOutputDev::restoreState(GfxState *state)
   m_codeToGIDStack.pop();
   m_rawFont = m_rawFontStack.top();
   m_rawFontStack.pop();
+  m_currentType3Font = m_type3FontStack.top();
+  m_type3FontStack.pop();
   m_currentBrush = m_currentBrushStack.top();
   m_currentBrushStack.pop();
   m_currentPen = m_currentPenStack.top();
@@ -372,8 +449,30 @@ void ArthurOutputDev::updateFont(GfxState *state)
     return;
   }
 
-  // is the font in the cache?
+  // The key to look in the font caches
   ArthurFontID fontID = {*gfxFont->getID(), state->getFontSize()};
+
+  // Current font is a type3 font
+  if (gfxFont->getType() == fontType3)
+  {
+    auto cacheEntry = m_type3FontCache.find(fontID);
+
+    if (cacheEntry!=m_type3FontCache.end()) {
+
+      // Take the font from the cache
+      m_currentType3Font = cacheEntry->second.get();
+
+    } else {
+
+      m_currentType3Font = new ArthurType3Font(m_doc, (Gfx8BitFont*)gfxFont);
+      m_type3FontCache.insert(std::make_pair(fontID,std::unique_ptr<ArthurType3Font>(m_currentType3Font)));
+
+    }
+
+    return;
+  }
+
+  // Non-type3: is the font in the cache?
   auto cacheEntry = m_rawFontCache.find(fontID);
 
   if (cacheEntry!=m_rawFontCache.end()) {
@@ -687,6 +786,132 @@ void ArthurOutputDev::eoFill(GfxState *state)
   m_painter.top()->fillPath( convertPath( state, state->getPath(), Qt::OddEvenFill ), m_currentBrush );
 }
 
+GBool ArthurOutputDev::axialShadedFill(GfxState *state, GfxAxialShading *shading, double tMin, double tMax)
+{
+  double x0, y0, x1, y1;
+  shading->getCoords(&x0, &y0, &x1, &y1);
+
+  // get the clip region bbox
+  double xMin, yMin, xMax, yMax;
+  state->getUserClipBBox(&xMin, &yMin, &xMax, &yMax);
+
+  // get the function domain
+  double t0 = shading->getDomain0();
+  double t1 = shading->getDomain1();
+
+  // Max number of splits along the t axis
+  constexpr int maxSplits = 256;
+
+  // Max delta allowed in any color component
+  const double colorDelta = (dblToCol(1 / 256.0));
+
+  // Number of color space components
+  auto nComps = shading->getColorSpace()->getNComps();
+
+  // Helper function to test two color objects for 'almost-equality'
+  auto isSameGfxColor = [&nComps,&colorDelta](const GfxColor &colorA, const GfxColor &colorB)
+                        {
+                          for (int k = 0; k < nComps; ++k) {
+                            if (abs(colorA.c[k] - colorB.c[k]) > colorDelta) {
+                              return false;
+                            }
+                          }
+                          return true;
+                        };
+
+  // Helper function: project a number into an interval
+  // With C++17 this is part of the standard library
+  auto clamp = [](double v, double lo, double hi)
+                  { return std::min(std::max(v,lo), hi); };
+
+  // ta stores all parameter values where we evaluate the input shading function.
+  // In between, QLinearGradient will interpolate linearly.
+  // We set up the array with three values.
+  std::array<double, maxSplits+1> ta;
+  ta[0] = tMin;
+  std::array<int, maxSplits+1> next;
+  next[0] = maxSplits / 2;
+  ta[maxSplits / 2] = 0.5 * (tMin + tMax);
+  next[maxSplits / 2] = maxSplits;
+  ta[maxSplits] = tMax;
+
+  // compute the color at t = tMin
+  double tt = clamp(t0 + (t1 - t0) * tMin, t0, t1);
+
+  GfxColor color0, color1;
+  shading->getColor(tt, &color0);
+
+  // Construct a gradient object and set its color at one parameter end
+  QLinearGradient gradient(QPointF(x0 + tMin * (x1 - x0), y0 + tMin * (y1 - y0)),
+                           QPointF(x0 + tMax * (x1 - x0), y0 + tMax * (y1 - y0)));
+
+  GfxRGB rgb;
+  shading->getColorSpace()->getRGB(&color0, &rgb);
+  QColor qColor(colToByte(rgb.r), colToByte(rgb.g), colToByte(rgb.b));
+  gradient.setColorAt(0,qColor);
+
+  // Look for more relevant parameter values by bisection
+  int i = 0;
+  while (i < maxSplits) {
+
+    int j = next[i];
+    while (j > i + 1) {
+
+      // Next parameter value to try
+      tt = clamp(t0 + (t1 - t0) * ta[j], t0, t1);
+      shading->getColor(tt, &color1);
+
+      // j is a good next color stop if the input shading can be approximated well
+      // on the interval (ta[i], ta[j]) by a linear interpolation.
+      // We test this by comparing the real color in the middle between ta[i] and ta[j]
+      // with the linear interpolant there.
+      auto midPoint = 0.5 * (ta[i] + ta[j]);
+      GfxColor colorAtMidPoint;
+      shading->getColor(midPoint, &colorAtMidPoint);
+
+      GfxColor linearlyInterpolatedColor;
+      for (int ii=0; ii<nComps; ii++)
+        linearlyInterpolatedColor.c[ii] = 0.5 * (color0.c[ii] + color1.c[ii]);
+
+      // If the two colors are equal, ta[j] is a good place for the next color stop; take it!
+      if (isSameGfxColor(colorAtMidPoint, linearlyInterpolatedColor))
+        break;
+
+      // Otherwise: bisect further
+      int k = (i + j) / 2;
+      ta[k] = midPoint;
+      next[i] = k;
+      next[k] = j;
+      j = k;
+    }
+
+    // set the color
+    GfxRGB rgb;
+    shading->getColorSpace()->getRGB(&color1, &rgb);
+    qColor.setRgb(colToByte(rgb.r), colToByte(rgb.g), colToByte(rgb.b));
+    gradient.setColorAt((ta[j] - tMin)/(tMax - tMin), qColor);
+
+    // Move to the next parameter region
+    color0 = color1;
+    i = next[i];
+  }
+
+  state->moveTo(xMin, yMin);
+  state->lineTo(xMin, yMax);
+  state->lineTo(xMax, yMax);
+  state->lineTo(xMax, yMin);
+  state->closePath();
+
+  // Actually paint the shaded region
+  QBrush newBrush(gradient);
+  m_painter.top()->fillPath( convertPath( state, state->getPath(), Qt::WindingFill ), newBrush );
+
+  state->clearPath();
+
+  // True means: The shaded region has been painted
+  return gTrue;
+}
+
 void ArthurOutputDev::clip(GfxState *state)
 {
   m_painter.top()->setClipPath(convertPath( state, state->getPath(), Qt::WindingFill ), Qt::IntersectClip );
@@ -702,6 +927,52 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
 			       double originX, double originY,
 			       CharCode code, int nBytes, Unicode *u, int uLen) {
 
+  // First handle type3 fonts
+  GfxFont *gfxFont = state->getFont();
+
+  GfxFontType fontType = gfxFont->getType();
+  if (fontType == fontType3) {
+
+    /////////////////////////////////////////////////////////////////////
+    //  Draw the QPicture that contains the glyph onto the page
+    /////////////////////////////////////////////////////////////////////
+
+    // Store the QPainter state; we need to modify it temporarily
+    m_painter.top()->save();
+
+    // Make the glyph position the coordinate origin -- that's our center of scaling
+    m_painter.top()->translate(QPointF(x-originX, y-originY));
+
+    const double* mat = gfxFont->getFontMatrix();
+    QTransform fontMatrix(mat[0], mat[1], mat[2], mat[3], mat[4], mat[5]);
+
+    // Scale with the font size
+    fontMatrix.scale(state->getFontSize(), state->getFontSize());
+    m_painter.top()->setTransform(fontMatrix,true);
+
+    // Apply the text matrix on top
+    const double *textMat = state->getTextMat();
+
+    QTransform textTransform(textMat[0] * state->getHorizScaling(),
+                             textMat[1] * state->getHorizScaling(),
+                             textMat[2],
+                             textMat[3],
+                             0,
+                             0);
+
+    m_painter.top()->setTransform(textTransform,true);
+
+    // Actually draw the glyph
+    int gid = m_currentType3Font->codeToGID[code];
+    m_painter.top()->drawPicture(QPointF(0,0), m_currentType3Font->getGlyph(gid));
+
+    // Restore transformation
+    m_painter.top()->restore();
+
+    return;
+  }
+
+
   // check for invisible text -- this is used by Acrobat Capture
   int render = state->getRender();
   if (render == 3 || !m_rawFont) {
@@ -768,17 +1039,6 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
   }
 }
 
-GBool ArthurOutputDev::beginType3Char(GfxState *state, double x, double y,
-				      double dx, double dy,
-				      CharCode code, Unicode *u, int uLen)
-{
-  return gFalse;
-}
-
-void ArthurOutputDev::endType3Char(GfxState *state)
-{
-}
-
 void ArthurOutputDev::type3D0(GfxState *state, double wx, double wy)
 {
 }
@@ -797,76 +1057,38 @@ void ArthurOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 				    int width, int height, GBool invert,
 				    GBool interpolate, GBool inlineImg)
 {
-  qDebug() << "drawImageMask";
-#if 0
-  unsigned char *buffer;
-  unsigned char *dest;
-  cairo_surface_t *image;
-  cairo_pattern_t *pattern;
-  int x, y;
-  ImageStream *imgStr;
-  Guchar *pix;
-  double *ctm;
-  cairo_matrix_t matrix;
-  int invert_bit;
-  int row_stride;
-
-  row_stride = (width + 3) & ~3;
-  buffer = (unsigned char *) malloc (height * row_stride);
-  if (buffer == nullptr) {
-    error(-1, "Unable to allocate memory for image.");
-    return;
-  }
-
-  /* TODO: Do we want to cache these? */
-  imgStr = new ImageStream(str, width, 1, 1);
+  std::unique_ptr<ImageStream> imgStr(new ImageStream(str, width,
+                                                      1,    // numPixelComps
+                                                      1));  // getBits
   imgStr->reset();
 
-  invert_bit = invert ? 1 : 0;
+  // TODO: Would using QImage::Format_Mono be more efficient here?
+  QImage image(width, height, QImage::Format_ARGB32);
+  unsigned int *data = (unsigned int *)image.bits();
+  int stride = image.bytesPerLine()/4;
 
-  for (y = 0; y < height; y++) {
-    pix = imgStr->getLine();
-    dest = buffer + y * row_stride;
-    for (x = 0; x < width; x++) {
+  QRgb fillColor = m_currentBrush.color().rgb();
+
+  for (int y = 0; y < height; y++) {
+
+    Guchar *pix = imgStr->getLine();
+
+    // Invert the vertical coordinate: y is increasing from top to bottom
+    // on the page, but y is increasing bottom to top in the picture.
+    unsigned int* dest = data + (height-1-y) * stride;
+
+    for (int x = 0; x < width; x++) {
+
+      bool opaque = ((bool)pix[x]) == invert;
+      dest[x] = (opaque) ? fillColor : 0;
 
-      if (pix[x] ^ invert_bit)
-	*dest++ = 0;
-      else
-	*dest++ = 255;
     }
   }
 
-  image = cairo_image_surface_create_for_data (buffer, CAIRO_FORMAT_A8,
-					  width, height, row_stride);
-  if (image == nullptr)
-    return;
-  pattern = cairo_pattern_create_for_surface (image);
-  if (pattern == nullptr)
-    return;
-
-  ctm = state->getCTM();
-  LOG (printf ("drawImageMask %dx%d, matrix: %f, %f, %f, %f, %f, %f\n",
-	       width, height, ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]));
-  matrix.xx = ctm[0] / width;
-  matrix.xy = -ctm[2] / height;
-  matrix.yx = ctm[1] / width;
-  matrix.yy = -ctm[3] / height;
-  matrix.x0 = ctm[2] + ctm[4];
-  matrix.y0 = ctm[3] + ctm[5];
-  cairo_matrix_invert (&matrix);
-  cairo_pattern_set_matrix (pattern, &matrix);
-
-  cairo_pattern_set_filter (pattern, CAIRO_FILTER_BEST);
-  /* FIXME: Doesn't the image mask support any colorspace? */
-  cairo_set_source_rgb (cairo, fill_color.r, fill_color.g, fill_color.b);
-  cairo_mask (cairo, pattern);
-
-  cairo_pattern_destroy (pattern);
-  cairo_surface_destroy (image);
-  free (buffer);
+  // At this point, the QPainter coordinate transformation (CTM) is such
+  // that QRect(0,0,1,1) is exactly the area of the image.
+  m_painter.top()->drawImage( QRect(0,0,1,1), image );
   imgStr->close ();
-  delete imgStr;
-#endif
 }
 
 //TODO: lots more work here.
